package com.jaxfront.demo;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.net.URL;

import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;
import javax.swing.UIManager;

import com.jaxfront.core.dom.DOMBuilder;
import com.jaxfront.core.dom.Document;
import com.jaxfront.core.images.SystemImages;
import com.jaxfront.core.util.JAXFrontProperties;
import com.jaxfront.core.util.LicenseErrorException;
import com.jaxfront.core.util.URLHelper;
import com.jaxfront.core.util.io.BrowserControl;
import com.jaxfront.pdf.PDFGenerator;
import com.jaxfront.swing.ui.editor.EditorPanel;
import com.jaxfront.swing.ui.editor.ShowXMLDialog;
import com.jaxfront.xuieditor.XUIEditor;

public class JAXFrontDemoSuite extends JFrame implements WindowListener {
	private final static String APPLICATION_TITLE = "JAXFront Demo Suite";
	private final static int WINDOW_HEIGHT = 600;
	private final static int WINDOW_WIDTH = 800;
	private final static int DIVIDER_LOCATION = 450;
	// My Example
	private static URL MY_EXAMPLE_XSD_URL;
	private static URL MY_EXAMPLE_XML_URL;
	private static URL MY_EXAMPLE_XUI_URL;
	private static String MY_EXAMPLE_ROOT_NAME;
	// Misc. Components
	private Document _currentDom;
	private JPanel _centerPanel;
	private JSplitPane _splitPane;
	private EditorPanel _editor;
	private String _currentLanguage = "en"; // english as default
	private ShowXMLDialog _xmlSyntaxDialog;
	// Actions
	private AbstractAction _germanLanguageAction;
	private AbstractAction _englishLanguageAction;
	private AbstractAction _frenchLanguageAction;
	private AbstractAction _serializeAction;
	private AbstractAction _printAction;
	private AbstractAction _validateAction;
	private AbstractAction _exitAction;
	private AbstractAction _myExampleAction;
	private AbstractAction _xuiEditorAction;

	public JAXFrontDemoSuite() {
		super();
		init();
	}

	private void init() {
		try {
			// init frame
			setTitle(APPLICATION_TITLE);
			setIconImage(JAXFrontProperties.getImage(SystemImages.ICON_XCENTRIC));
			getContentPane().setLayout(new BorderLayout());
			// initialize actions and gui components
			initActions();
			initMenuBar();
			initToolBar();
			_centerPanel = new JPanel(new BorderLayout());
			_centerPanel.setBorder(null);
			_splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
			_splitPane.setDividerSize(4);
			_splitPane.setBorder(null);
			_splitPane.setTopComponent(_centerPanel);
			getContentPane().add(_splitPane, BorderLayout.CENTER);
			addWindowListener(this);
			buildDemoSuite();
			setSize(WINDOW_WIDTH, WINDOW_HEIGHT);
			setVisible(true);
		} catch (LicenseErrorException licEx) {
			licEx.showLicenseDialog(this);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel("com.jgoodies.plaf.plastic.PlasticXPLookAndFeel");
		} catch (Throwable t) {}
		for (int a = 0; a < args.length; a++) {
			String arg = args[a];
			if (arg.endsWith("xsd")) {
				MY_EXAMPLE_XSD_URL = URLHelper.getUserURL(arg);
			} else if (arg.endsWith("xml")) {
				MY_EXAMPLE_XML_URL = URLHelper.getUserURL(arg);
			} else if (arg.endsWith("xui")) {
				MY_EXAMPLE_XUI_URL = URLHelper.getUserURL(arg);
			} else {
				MY_EXAMPLE_ROOT_NAME = arg;
			}
		}
		new JAXFrontDemoSuite();
	}

	private void initActions() {
		_germanLanguageAction = new AbstractAction("German", JAXFrontProperties.getImageIcon(SystemImages.ICON_XUI_DEFINITION_GLOBAL)) {
			public void actionPerformed(ActionEvent e) {
				setLanguage("de");
			}
		};
		_englishLanguageAction = new AbstractAction("English", JAXFrontProperties.getImageIcon(SystemImages.ICON_XUI_DEFINITION_GLOBAL)) {
			public void actionPerformed(ActionEvent e) {
				setLanguage("en");
			}
		};
		_frenchLanguageAction = new AbstractAction("French", JAXFrontProperties.getImageIcon(SystemImages.ICON_XUI_DEFINITION_GLOBAL)) {
			public void actionPerformed(ActionEvent e) {
				setLanguage("fr");
			}
		};
		_serializeAction = new AbstractAction("Serialize", JAXFrontProperties.getImageIcon(SystemImages.ICON_EDIT_XML)) {
			public void actionPerformed(ActionEvent e) {
				serialize();
			}
		};
		_myExampleAction = new AbstractAction("My own example", JAXFrontProperties.getImageIcon(SystemImages.ICON_NEW_FILE)) {
			public void actionPerformed(ActionEvent e) {
				myExample();
			}
		};
		_printAction = new AbstractAction("Print PDF", JAXFrontProperties.getImageIcon(SystemImages.ICON_PDF)) {
			public void actionPerformed(ActionEvent e) {
				print();
			}
		};
		_validateAction = new AbstractAction("Validate", JAXFrontProperties.getImageIcon(SystemImages.ICON_VALIDATE)) {
			public void actionPerformed(ActionEvent e) {
				validateXML();
			}
		};
		_exitAction = new AbstractAction("Exit", JAXFrontProperties.getImageIcon(SystemImages.ICON_CLOSE)) {
			public void actionPerformed(ActionEvent e) {
				exit();
			}
		};
		_xuiEditorAction = new AbstractAction("Open XUIEditor", JAXFrontProperties.getImageIcon(SystemImages.ICON_XCENTRIC)) {
			public void actionPerformed(ActionEvent e) {
				openXUIEditor();
			}
		};
		_englishLanguageAction.setEnabled(false);
		_germanLanguageAction.setEnabled(false);
		_frenchLanguageAction.setEnabled(false);
		_serializeAction.setEnabled(false);
		_printAction.setEnabled(false);
		_validateAction.setEnabled(false);
	}

	private void myExample() {
		try {
			if (_currentDom == null) {
				_currentDom = DOMBuilder.getInstance().build(null, MY_EXAMPLE_XSD_URL, MY_EXAMPLE_XML_URL, MY_EXAMPLE_XUI_URL, MY_EXAMPLE_ROOT_NAME);
				_currentDom.getGlobalDefinition().setIsUsingButtonBar(false);
				_currentDom.getGlobalDefinition().setIsUsingStatusBar(false);
			}
			if (_editor != null) _editor.selectNode((com.jaxfront.core.type.Type) null);
			visualizeDOM();
			setApplicationTitle("My Example");
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	private void initMenuBar() {
		JMenuBar menuBar = new JMenuBar();
		JMenu applicationMenu = new JMenu("Application");
		applicationMenu.add(_exitAction);
		JMenu editMenu = new JMenu("Edit");
		editMenu.add(_serializeAction);
		editMenu.add(_validateAction);
		JMenu languageMenu = new JMenu("Language");
		languageMenu.add(_englishLanguageAction);
		languageMenu.add(_germanLanguageAction);
		languageMenu.add(_frenchLanguageAction);
		menuBar.add(applicationMenu);
		if (MY_EXAMPLE_XSD_URL != null) {
			JMenu myExampleMenu = new JMenu("My Example");
			myExampleMenu.add(_myExampleAction);
			menuBar.add(myExampleMenu);
		}
		menuBar.add(editMenu);
		menuBar.add(languageMenu);
		setJMenuBar(menuBar);
	}

	private void initToolBar() {
		JToolBar toolBar = new JToolBar();
		toolBar.add(_serializeAction);
		toolBar.add(_validateAction);
		toolBar.addSeparator();
		toolBar.add(_xuiEditorAction);
		toolBar.addSeparator();
		toolBar.add(_printAction);
		getContentPane().add(toolBar, BorderLayout.NORTH);
	}

	private void validateXML() {
		if (_currentDom != null) {
			_currentDom.validate();
		}
	}

	private void print() {
		if (_currentDom != null) {
			ByteArrayOutputStream bos = PDFGenerator.getInstance().print(_currentDom);
			if (bos != null) {
				try {
					String tempPDFName = "c:\\temp\\test.pdf";
					FileOutputStream fos = new FileOutputStream(tempPDFName);
					bos.writeTo(fos);
					fos.close();
					BrowserControl.displayURL(tempPDFName);
				} catch (Throwable t) {
					t.printStackTrace();
				}
			}
		}
	}

	private String getCurrentLanguage() {
		return _currentLanguage;
	}

	private void visualizeDOM() {
		// set language
		_currentDom.getGlobalDefinition().setLanguage(getCurrentLanguage());
		EditorPanel editorPanel = new EditorPanel(_currentDom.getRootType(), this);
		editorPanel.setBorder(null);
		editorPanel.setHelpPanelPosition(BorderLayout.NORTH);
		JPanel validationErrorPanel = new JPanel(new BorderLayout());
		validationErrorPanel.setBorder(null);
		editorPanel.setTargetMessageTable(validationErrorPanel);
		if (_editor != null) _centerPanel.remove(_editor);
		_editor = editorPanel;
		_centerPanel.add(_editor, BorderLayout.CENTER);
		int lastDividerLocation = DIVIDER_LOCATION;
		if (_splitPane.getBottomComponent() != null) {
			lastDividerLocation = _splitPane.getDividerLocation();
			_splitPane.remove(_splitPane.getBottomComponent());
		}
		_splitPane.setBottomComponent(validationErrorPanel);
		_splitPane.setDividerLocation(lastDividerLocation);
		_splitPane.validate();
		// activate actions
		_englishLanguageAction.setEnabled(true);
		_germanLanguageAction.setEnabled(true);
		_frenchLanguageAction.setEnabled(true);
		_serializeAction.setEnabled(true);
		_printAction.setEnabled(true);
		_validateAction.setEnabled(true);
	}

	private void setLanguage(String language) {
		_currentLanguage = language;
		visualizeDOM();
	}

	private void serialize() {
		if (_xmlSyntaxDialog == null) _xmlSyntaxDialog = new ShowXMLDialog(_currentDom);
		else _xmlSyntaxDialog.setDom(_currentDom);
		_xmlSyntaxDialog.setSize(600, 400);
		_xmlSyntaxDialog.prettyPrint();
		Dimension dialogDim = _xmlSyntaxDialog.getSize();
		Dimension thisDim = getSize();
		int x = (thisDim.width - dialogDim.width) / 2;
		int y = (thisDim.height - dialogDim.height) / 2;
		if (getLocation().x > 0 || getLocation().y > 0) {
			x = x + getLocation().x;
			y = y + getLocation().y;
		}
		_xmlSyntaxDialog.setLocation(((x > 0) ? x : 0), ((y > 0) ? y : 0));
		_xmlSyntaxDialog.setVisible(true);
	}

	private void setApplicationTitle(String currentFileName) {
		setTitle(APPLICATION_TITLE + " - " + currentFileName);
	}

	private void buildDemoSuite() {
		URL url = URLHelper.getUserURL("examples/demo-suite/jaxfront-test-schema.xsd");
		URL xmlUrl = URLHelper.getUserURL("examples/demo-suite/jaxfront-test-schema.xml");
		URL xuiUrl = URLHelper.getUserURL("examples/demo-suite/jaxfront-test-schema.xui");
		String root = "root";
		try {
			if (_currentDom == null) {
				_currentDom = DOMBuilder.getInstance().build(null, url, xmlUrl, xuiUrl, root);
				_currentDom.getGlobalDefinition().setIsUsingStatusBar(false);
			}
			visualizeDOM();
		} catch (Exception ex) {}
	}

	private void exit() {
		System.exit(0);
	}

	public void windowActivated(WindowEvent e) {}

	public void windowClosed(WindowEvent e) {
		exit();
	}

	public void windowClosing(WindowEvent e) {
		exit();
	}

	public void windowDeactivated(WindowEvent e) {}

	public void windowDeiconified(WindowEvent e) {}

	public void windowIconified(WindowEvent e) {}

	public void windowOpened(WindowEvent e) {}

	public void openXUIEditor() {
		String[] args = new String[] { "examples/demo-suite/jaxfront-test-schema.xsd", "-ROOT=root", "-XML=examples/demo-suite/jaxfront-test-schema.xml", "-XUI=examples/demo-suite/jaxfront-test-schema.xui" };
		XUIEditor.main(args);
	}
}
