package com.jaxfront.demo;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.net.URL;

import javax.swing.AbstractAction;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.UIManager;

import com.jaxfront.core.dom.DOMBuilder;
import com.jaxfront.core.dom.Document;
import com.jaxfront.core.help.HelpEvent;
import com.jaxfront.core.help.HelpListener;
import com.jaxfront.core.images.SystemImages;
import com.jaxfront.core.ui.TypeVisualizerFactory;
import com.jaxfront.core.util.JAXFrontProperties;
import com.jaxfront.core.util.LicenseErrorException;
import com.jaxfront.core.util.URLHelper;
import com.jaxfront.core.util.io.BrowserControl;
import com.jaxfront.core.util.io.cache.XUICache;
import com.jaxfront.pdf.PDFGenerator;
import com.jaxfront.swing.ui.beans.TypeTable;
import com.jaxfront.swing.ui.editor.EditorPanel;
import com.jaxfront.swing.ui.editor.ShowXMLDialog;

/*****************************************************************************************************************************************************
 * JAXFront Integration Example and use of the EditorPanel API.
 ****************************************************************************************************************************************************/
/*-- 

 Copyright (C) 2001-2012 by xcentric technology & consulting GmbH. All rights reserved.

 This software is the confidential and proprietary information of xcentric technology & 
 consulting GmbH ("Confidential Information"). You shall not disclose such Confidential 
 Information and shall use it only in accordance with the terms of the license agreement
 you entered into with xcentric.

 www.jaxfront.com

 */
public class TestFrame extends JFrame implements WindowListener, HelpListener {
	private final static String APPLICATION_TITLE = "JAXFront Demo";
	private final static int WINDOW_HEIGHT = 768;
	private final static int WINDOW_WIDTH = 1024;
	private static String _title;
	// Misc. Components
	private JTabbedPane _tabbedPane;
	private JPanel _helpPanel;
	private JPanel _errorHolderPanel;
	private JPanel _unlockedFieldsPanel;
	private Document _currentDom;
	private JPanel _centerPanel;
	private JSplitPane _splitPane;
	private EditorPanel _editor;
	private String _currentLanguage = "en"; // english as default
	// Actions
	private AbstractAction _germanLanguageAction;
	private AbstractAction _englishLanguageAction;
	private AbstractAction _frenchLanguageAction;
	private AbstractAction _italianLanguageAction;
	private AbstractAction _serializeAction;
	private AbstractAction _example1Action;
	private AbstractAction _example2Action;
	private AbstractAction _example3Action;
	private AbstractAction _printAction;
	private AbstractAction _validateAction;
	private AbstractAction _lockAction;
	private AbstractAction _unLockAction;
	private AbstractAction _exitAction;
	private AbstractAction _createNewAction;
	private AbstractAction _releaseXUICache;

	public TestFrame() {
		super();
		init();
	}

	private void init() {
		try {
			// init frame
			setTitle(APPLICATION_TITLE);
			setIconImage(JAXFrontProperties.getImage(SystemImages.ICON_XCENTRIC));
			getContentPane().setLayout(new BorderLayout());
			// initialize actions and gui components
			initActions();
			initMenuBar();
			initToolBar();
			_centerPanel = new JPanel(new BorderLayout());
			_centerPanel.setBorder(null);
			_tabbedPane = new JTabbedPane();
			_helpPanel = new JPanel();
			_errorHolderPanel = new JPanel(new BorderLayout());
			_unlockedFieldsPanel = new JPanel(new BorderLayout());
			_tabbedPane.addTab("Hints & Errors", _errorHolderPanel);
			_tabbedPane.addTab("Help", _helpPanel);
			_tabbedPane.addTab("Lock", _unlockedFieldsPanel);
			_splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
			_splitPane.setDividerSize(4);
			_splitPane.setDividerLocation(500);
			_splitPane.setBorder(null);
			_splitPane.setTopComponent(_centerPanel);
			_splitPane.setBottomComponent(_tabbedPane);
			getContentPane().add(_splitPane, BorderLayout.CENTER);
			addWindowListener(this);
			if (_title == null) setTitle("JAXFront Demo Example");
			else setTitle(_title);
			setSize(WINDOW_WIDTH, WINDOW_HEIGHT);
			setVisible(true);
		} catch (LicenseErrorException licEx) {
			licEx.showLicenseDialog(this);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel("com.jgoodies.looks.windows.WindowsLookAndFeel");
		} catch (Throwable t) {}
		new TestFrame();
	}

	private void initActions() {
		_releaseXUICache = new AbstractAction("Release XUI Cache", JAXFrontProperties.getImageIcon(SystemImages.ICON_DELETE)) {
			public void actionPerformed(ActionEvent e) {
				releaseXUICache();
			}
		};
		_germanLanguageAction = new AbstractAction("German", JAXFrontProperties.getImageIcon(SystemImages.ICON_XUI_DEFINITION_GLOBAL)) {
			public void actionPerformed(ActionEvent e) {
				setLanguage("de");
			}
		};
		_englishLanguageAction = new AbstractAction("English", JAXFrontProperties.getImageIcon(SystemImages.ICON_XUI_DEFINITION_GLOBAL)) {
			public void actionPerformed(ActionEvent e) {
				setLanguage("en");
			}
		};
		_frenchLanguageAction = new AbstractAction("French", JAXFrontProperties.getImageIcon(SystemImages.ICON_XUI_DEFINITION_GLOBAL)) {
			public void actionPerformed(ActionEvent e) {
				setLanguage("fr");
			}
		};
		_italianLanguageAction = new AbstractAction("Italian", JAXFrontProperties.getImageIcon(SystemImages.ICON_XUI_DEFINITION_GLOBAL)) {
			public void actionPerformed(ActionEvent e) {
				setLanguage("it");
			}
		};
		_serializeAction = new AbstractAction("Serialize", JAXFrontProperties.getImageIcon(SystemImages.ICON_EDIT_XML)) {
			public void actionPerformed(ActionEvent e) {
				serialize();
			}
		};
		_lockAction = new AbstractAction("Lock", JAXFrontProperties.getImageIcon(SystemImages.ICON_LOCK)) {
			public void actionPerformed(ActionEvent e) {
				lock();
			}
		};
		_unLockAction = new AbstractAction("Unlock", JAXFrontProperties.getImageIcon(SystemImages.ICON_UNLOCK)) {
			public void actionPerformed(ActionEvent e) {
				unLock();
			}
		};
		_example1Action = new AbstractAction("Example 1", JAXFrontProperties.getImageIcon("images/xsd.gif")) {
			public void actionPerformed(ActionEvent e) {
				example1();
			}
		};
		_example2Action = new AbstractAction("Example 2 (xsd + xml)", JAXFrontProperties.getImageIcon("images/xml.gif")) {
			public void actionPerformed(ActionEvent e) {
				example2();
			}
		};
		_example3Action = new AbstractAction("Example 3 (xsd + xml + xui)", JAXFrontProperties.getImageIcon("images/xui.gif")) {
			public void actionPerformed(ActionEvent e) {
				example3(true);
			}
		};
		_printAction = new AbstractAction("Print PDF", JAXFrontProperties.getImageIcon(SystemImages.ICON_PDF)) {
			public void actionPerformed(ActionEvent e) {
				print();
			}
		};
		_validateAction = new AbstractAction("Validate", JAXFrontProperties.getImageIcon(SystemImages.ICON_VALIDATE)) {
			public void actionPerformed(ActionEvent e) {
				validateXML();
			}
		};
		_exitAction = new AbstractAction("Exit", JAXFrontProperties.getImageIcon(SystemImages.ICON_CLOSE)) {
			public void actionPerformed(ActionEvent e) {
				exit();
			}
		};
		_createNewAction = new AbstractAction("Create New", JAXFrontProperties.getImageIcon(SystemImages.ICON_NEW)) {
			public void actionPerformed(ActionEvent e) {
				example3(false);
			}
		};
		_englishLanguageAction.setEnabled(false);
		_germanLanguageAction.setEnabled(false);
		_frenchLanguageAction.setEnabled(false);
		_serializeAction.setEnabled(false);
		_printAction.setEnabled(false);
		_validateAction.setEnabled(false);
	}

	private void initMenuBar() {
		JMenuBar menuBar = new JMenuBar();
		JMenu applicationMenu = new JMenu("Application");
		applicationMenu.add(_releaseXUICache);
		applicationMenu.add(_exitAction);
		JMenu exampleMenu = new JMenu("Examples");
		exampleMenu.addSeparator();
		exampleMenu.add(_example1Action);
		exampleMenu.add(_example2Action);
		exampleMenu.add(_example3Action);
		JMenu editMenu = new JMenu("Edit");
		editMenu.add(_createNewAction);
		editMenu.addSeparator();
		editMenu.add(_serializeAction);
		editMenu.add(_validateAction);
		editMenu.addSeparator();
		editMenu.add(_lockAction);
		editMenu.add(_unLockAction);
		editMenu.addSeparator();
		JMenu languageMenu = new JMenu("Language");
		languageMenu.add(_englishLanguageAction);
		languageMenu.add(_germanLanguageAction);
		languageMenu.add(_frenchLanguageAction);
		languageMenu.add(_italianLanguageAction);
		menuBar.add(applicationMenu);
		menuBar.add(exampleMenu);
		menuBar.add(editMenu);
		menuBar.add(languageMenu);
		setJMenuBar(menuBar);
	}

	private void initToolBar() {
		JToolBar toolBar = new JToolBar();
		toolBar.add(_example1Action);
		toolBar.add(_example2Action);
		toolBar.add(_example3Action);
		toolBar.addSeparator();
		toolBar.add(_createNewAction);
		toolBar.add(_serializeAction);
		toolBar.add(_validateAction);
		toolBar.addSeparator();
		toolBar.add(_printAction);
		toolBar.addSeparator();
		JLabel logoLabel = new JLabel(JAXFrontProperties.getImageIcon("images/jaxfront.gif"));
		JPanel logoPanel = new JPanel(new BorderLayout());
		logoPanel.add(logoLabel, BorderLayout.EAST);
		toolBar.add(logoPanel);
		getContentPane().add(toolBar, BorderLayout.NORTH);
	}

	private void validateXML() {
		if (_currentDom != null) {
			_currentDom.validate();
		}
	}

	private void print() {
		if (_currentDom != null) {
			ByteArrayOutputStream bos = PDFGenerator.getInstance().print(_currentDom);
			if (bos != null) {
				try {
					String tempPDFName = "c:\\temp\\test.pdf";
					FileOutputStream fos = new FileOutputStream(tempPDFName);
					bos.writeTo(fos);
					fos.close();
					BrowserControl.displayURL(tempPDFName);
				} catch (Throwable t) {
					t.printStackTrace();
				}
			}
		}
	}

	private String getCurrentLanguage() {
		return _currentLanguage;
	}

	private void visualizeDOM() {
		// set language
		_unlockedFieldsPanel.removeAll();
		_unlockedFieldsPanel.updateUI();
		_currentDom.getGlobalDefinition().setLanguage(getCurrentLanguage());
		com.jaxfront.core.type.Type lastSelectedType = null;
		if (_editor != null && _editor.getSelectedTreeNode() != null) {
			lastSelectedType = _editor.getSelectedTreeNode().getType();
		}
		TypeVisualizerFactory.getInstance().releaseCache(_currentDom);
		EditorPanel editorPanel = new EditorPanel(_currentDom.getRootType(), this);
		if (lastSelectedType != null) {
			editorPanel.selectNode(lastSelectedType);
		}
		editorPanel.setBorder(null);
		editorPanel.addHelpListener(this);
		JPanel validationErrorPanel = new JPanel(new BorderLayout());
		validationErrorPanel.setBorder(null);
		editorPanel.setTargetMessageTable(validationErrorPanel);
		if (_editor != null) _centerPanel.remove(_editor);
		_editor = editorPanel;
		_centerPanel.add(_editor, BorderLayout.CENTER);
		if (_splitPane.getBottomComponent() != null) {
			// lastDividerLocation = _splitPane.getDividerLocation();
			_errorHolderPanel.removeAll();
			_errorHolderPanel.add(validationErrorPanel, BorderLayout.CENTER);
			_splitPane.validate();
		}
		// activate actions
		_englishLanguageAction.setEnabled(true);
		_germanLanguageAction.setEnabled(true);
		_frenchLanguageAction.setEnabled(true);
		_serializeAction.setEnabled(true);
		_printAction.setEnabled(true);
		_validateAction.setEnabled(true);
	}

	private void setLanguage(String language) {
		_currentLanguage = language;
		visualizeDOM();
	}

	private void serialize() {
		ShowXMLDialog dialog = new ShowXMLDialog(_currentDom);
		dialog.prettyPrint();
		Dimension dialogDim = dialog.getSize();
		Dimension thisDim = getSize();
		int x = (thisDim.width - dialogDim.width) / 2;
		int y = (thisDim.height - dialogDim.height) / 2;
		if (getLocation().x > 0 || getLocation().y > 0) {
			x = x + getLocation().x;
			y = y + getLocation().y;
		}
		dialog.setLocation(((x > 0) ? x : 0), ((y > 0) ? y : 0));
		dialog.setVisible(true);
	}

	private void lock() {
		String[] fieldToUnlock = new String[] { "/SchadenMeldung/Ereignis/Hergang", "/SchadenMeldung/WeiteresVorgehenNotizen[list]" };
		if (_currentDom != null && _currentDom.getRootType().getQName().equals("purchaseOrder")) {
			// fieldToUnlock = new String[] {"/purchaseOrder/@orderDate", "/purchaseOrder/item[list]"};
			fieldToUnlock = new String[] { "/purchaseOrder/@orderDate", "/purchaseOrder/paymentMethod" };
		}
		_currentDom.lock(fieldToUnlock);
		_unlockedFieldsPanel.removeAll();
		_unlockedFieldsPanel.setFocusable(false);
		JScrollPane scrollPane = new JScrollPane(new TypeTable(_currentDom, fieldToUnlock));
		scrollPane.setFocusable(false);
		scrollPane.getViewport().setFocusable(false);
		scrollPane.getVerticalScrollBar().setFocusable(false);
		scrollPane.getHorizontalScrollBar().setFocusable(false);
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		_unlockedFieldsPanel.add(scrollPane, BorderLayout.CENTER);
		_unlockedFieldsPanel.updateUI();
		_tabbedPane.setSelectedIndex(2);
	}

	private void unLock() {
		_currentDom.unlock();
		_unlockedFieldsPanel.removeAll();
		_unlockedFieldsPanel.updateUI();
	}

	private void setApplicationTitle(String currentFileName) {
		setTitle(APPLICATION_TITLE + " - " + currentFileName);
	}

	private void example1() {
		URL url = URLHelper.getUserURL("examples/purchaseOrder/po.xsd");
		URL xmlUrl = null;
		URL xuiUrl = null;
		try {
			_currentDom = DOMBuilder.getInstance().build("default-context", url, xmlUrl, xuiUrl, null);
			_currentDom.getGlobalDefinition().setIsUsingButtonBar(false);
			_currentDom.getGlobalDefinition().setIsUsingStatusBar(false);
			_currentDom.getGlobalDefinition().setLanguage(_currentLanguage);
			if (_editor != null) _editor.selectNode((com.jaxfront.core.type.Type) null);
			visualizeDOM();
			setApplicationTitle("Default Rendering (based on XML Schema only)");
		} catch (Exception ex) {
			ex.printStackTrace();
			String xsd = null;
			String xml = null;
			String xui = null;
			if (url != null) xsd = url.toExternalForm();
			if (xmlUrl != null) xml = xmlUrl.toExternalForm();
			if (xuiUrl != null) xui = xuiUrl.toExternalForm();
			JOptionPane.showConfirmDialog(this, "Unable to visualize: \n " + "xsd - " + xsd + "\n" + "xml - " + xml + "\n" + "xui - " + xui + "\n");
		}
	}

	private void example2() {
		URL url = URLHelper.getUserURL("examples/purchaseOrder/po.xsd");
		URL xmlUrl = URLHelper.getUserURL("examples/purchaseOrder/po.xml");
		URL xuiUrl = null;
		String root = null;
		try {
			_currentDom = DOMBuilder.getInstance().build("default-context", url, xmlUrl, xuiUrl, root);
			_currentDom.getGlobalDefinition().setIsUsingButtonBar(false);
			_currentDom.getGlobalDefinition().setIsUsingStatusBar(false);
			_currentDom.getGlobalDefinition().setLanguage(_currentLanguage);
			if (_editor != null) _editor.selectNode((com.jaxfront.core.type.Type) null);
			visualizeDOM();
			setApplicationTitle("Default Rendering including XML data (based on XML Schema only)");
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	private void example3(boolean useXML) {
		URL url = URLHelper.getUserURL("examples/purchaseOrder/po.xsd");
		URL xmlUrl = URLHelper.getUserURL("examples/purchaseOrder/po.xml");
		URL xuiUrl = URLHelper.getUserURL("examples/purchaseOrder/po.xui");
		String root = null;
		if (!useXML) {
			xmlUrl = null;
		}
		try {
			_currentDom = DOMBuilder.getInstance().build("default-context", url, xmlUrl, xuiUrl, root);
			_currentDom.getGlobalDefinition().setLanguage(_currentLanguage);
			if (_editor != null) _editor.selectNode((com.jaxfront.core.type.Type) null);
			visualizeDOM();
			setApplicationTitle("Individual Rendering (based on XUI)");
		} catch (Exception ex) {}
	}

	private void exit() {
		System.exit(0);
	}

	public void windowActivated(WindowEvent e) {}

	public void windowClosed(WindowEvent e) {
		exit();
	}

	public void windowClosing(WindowEvent e) {
		exit();
	}

	public void windowDeactivated(WindowEvent e) {}

	public void windowDeiconified(WindowEvent e) {}

	public void windowIconified(WindowEvent e) {}

	public void windowOpened(WindowEvent e) {}

	public void showHelp(HelpEvent event) {
		_helpPanel.removeAll();
		String helpText = "Help (" + event.getHelpID() + "): " + ((com.jaxfront.core.type.Type) event.getSource()).getXPathLocation();
		JLabel helpLabel = new JLabel(helpText);
		_helpPanel.add(helpLabel);
		_helpPanel.updateUI();
	}

	private void releaseXUICache() {
		XUICache.getInstance().releaseCache();
	}
}