package com.jaxfront.demo.api;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.net.URL;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import com.jaxfront.core.dom.DOMBuilder;
import com.jaxfront.core.dom.Document;
import com.jaxfront.core.dom.DocumentCreationException;
import com.jaxfront.core.log.LogRegistry;
import com.jaxfront.core.type.Type;
import com.jaxfront.core.ui.TypeVisualizerFactory;
import com.jaxfront.core.util.URLHelper;
import com.jaxfront.swing.ui.visualizers.AbstractCompositeView;
/**
 * Demonstrates how to use the JAXFront VisualizerFactory to place
 * components on your own layout implementation.
 */
/*-- 

Copyright (C) 2001 by xcentric technology & consulting GmbH.
All rights reserved.

This software is the confidential and proprietary information
of xcentric technology & consulting GmbH ("Confidential Information"). 
You shall not disclose such Confidential Information and shall use
it only in accordance with the terms of the license agreement
you entered into with xcentric.

www.jaxfront.com

*/
public class OwnLayoutManager extends AbstractCompositeView implements ListSelectionListener{
	
	private Document _dom;
	private JPanel _detailPanel;
	
	protected void defaultBuild() {
		setLayout(new BorderLayout());
		try {
	        URL xsdURL = URLHelper.getUserURL("examples/purchaseOrder/po.xsd");
	        URL xmlURL = URLHelper.getUserURL("examples/purchaseOrder/po.xml");
	        URL xuiURL = URLHelper.getUserURL("examples/purchaseOrder/po.xui");
			_dom = DOMBuilder.getInstance().build(null, xsdURL, xmlURL, xuiURL, "purchaseOrder");
			
			JPanel upperPanel = new JPanel(new GridBagLayout());
			GridBagConstraints c = new GridBagConstraints();
			c.fill = GridBagConstraints.HORIZONTAL;
			
			Type purchaseOrderRoot = _dom.getRootType();
			
			JComponent visualizer = (JComponent)TypeVisualizerFactory.getInstance().getVisualizer(purchaseOrderRoot.getDirectChild("orderDate"));
			c.weightx = 0.5;
			c.gridx = 0;
			c.gridy = 0;
			upperPanel.add(visualizer, c);
			
			visualizer = (JComponent)TypeVisualizerFactory.getInstance().getVisualizer(purchaseOrderRoot.getDirectChild("deliveryOption"));
			c.gridx = 1;
			c.gridy = 0;
			upperPanel.add(visualizer, c);

			visualizer = (JComponent)TypeVisualizerFactory.getInstance().getVisualizer(purchaseOrderRoot.getDirectChild("email"));
			c.gridx = 0;
			c.gridy = 1;
			upperPanel.add(visualizer, c);

			JPanel lowerPanel = new JPanel(new BorderLayout());
			_detailPanel = new JPanel(new BorderLayout());
			JList list = new JList(new String[]{"shipTo", "billTo", "paymentMethod", "item"});
			list.addListSelectionListener(this);
			list.setBorder(BorderFactory.createEtchedBorder());
			JScrollPane scrollPane = new JScrollPane(list);
			scrollPane.setPreferredSize(new Dimension(200,200));
			JPanel listPanel = new JPanel(new BorderLayout());
			listPanel.add(scrollPane, BorderLayout.NORTH);
			lowerPanel.add(listPanel, BorderLayout.WEST);
			lowerPanel.add(_detailPanel, BorderLayout.CENTER);
			
			add(upperPanel, BorderLayout.NORTH);
			add(lowerPanel, BorderLayout.CENTER);
			
			
		}
		catch (DocumentCreationException ex) {
			LogRegistry.getInstance().warn(getClass(), "Error while creating po.xsd DOM: " + ex.getLocalizedMessage());		
		}
		
	}

	public void setApplicationRequired(boolean aBoolean) {}

	public void setRequired(boolean aBoolean) {}

	public void valueChanged(ListSelectionEvent e) {
		_detailPanel.removeAll();
		String name = (String)((JList)e.getSource()).getSelectedValue();
		JComponent visualizer = (JComponent)TypeVisualizerFactory.getInstance().getVisualizer(_dom.getRootType().getDirectChild(name));
		_detailPanel.add(visualizer, BorderLayout.CENTER);
		_detailPanel.updateUI();
		
	}
}
