package com.jaxfront.demo;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.*;
import com.jaxfront.core.dom.*;
import com.jaxfront.core.schema.SchemaCreationException;
import com.jaxfront.core.schema.ValidationException;
import com.jaxfront.core.ui.TypeVisualizerFactory;
import com.jaxfront.core.util.URLHelper;
import com.jaxfront.swing.ui.editor.EditorPanel;

/**
 * Demonstrates the use of JAXFront UI's within an internal JFrame.
 * 
 */
/*-- 

 Copyright (C) 2001-2003 by xcentric technology & consulting GmbH. All rights reserved.

 This software is the confidential and proprietary information of xcentric technology & 
 consulting GmbH ("Confidential Information"). You shall not disclose such Confidential 
 Information and shall use it only in accordance with the terms of the license agreement
 you entered into with xcentric.

 www.jaxfront.com

 */
public class JIFrameDemo  {
	private static JInternalFrame _currentIFrame;
	private static EditorPanel _currentEditorPanel;
	
	private static Document _dom;

	/* Main View */
	public static void main(String[] a) {
		String lnfName = "com.jgoodies.looks.plastic.PlasticXPLookAndFeel";
		try {
			UIManager.setLookAndFeel(lnfName);
		} catch (UnsupportedLookAndFeelException ex1) {
			System.err.println("Unsupported LookAndFeel: " + lnfName);
		} catch (ClassNotFoundException ex2) {
			System.err.println("LookAndFeel class not found: " + lnfName);
		} catch (InstantiationException ex3) {
			System.err.println("Could not load LookAndFeel: " + lnfName);
		} catch (IllegalAccessException ex4) {
			System.err.println("Cannot use LookAndFeel: " + lnfName);
		}
		final JFrame jf = new JFrame("JIFrameDemo Main Window");
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		screenSize.width -= 300;
		screenSize.height -= 300;
		jf.setSize(screenSize);
		jf.setLocation(20, 20);
		JMenuBar mb = new JMenuBar();
		jf.setJMenuBar(mb);
		JMenu fm = new JMenu("File");
		mb.add(fm);
		JMenuItem mi;
		JMenuItem mi2;
		fm.add(mi2 = new JMenuItem("Save"));
		fm.add(mi = new JMenuItem("Exit"));
		mi2.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				((JMenuItem)e.getSource()).transferFocus();
				saveDOM();
			}
		});
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				System.exit(0);
			}
		});

		JMenu languageMenu = new JMenu("Language");
		mb.add(languageMenu);
		addLanugages(languageMenu);
		
		JButton testButton = new JButton("test");
		mb.add(testButton);

		
		JDesktopPane dtp = new JDesktopPane();
		jf.setContentPane(dtp);
		_currentIFrame = new JInternalFrame("JAXFront Demo (EditorPanel)", true, true, true, true);
		try {
			_dom = DOMBuilder.getInstance().build(null, URLHelper.getUserURL("examples/purchaseOrder/po.xsd"), URLHelper.getUserURL("examples/purchaseOrder/po.xml"),
					URLHelper.getUserURL("examples/purchaseOrder/po.xui"), null);
			_currentEditorPanel = new EditorPanel(_dom.getRootType(), null);
			_currentIFrame.setContentPane(_currentEditorPanel);
			_currentIFrame.setSize(650, 400);
			_currentIFrame.setLocation(10, 10);
			_currentIFrame.setVisible(true);
			dtp.add(_currentIFrame);
			_dom.addDirtyChangeListener(new DirtyChangeListener() {
			      public void dirtyChange(DirtyChangeEvent e) {
			  		String title = "JAXFront Demo (EditorPanel)";
					if (e.hasChanged()) {
						title += "*";
					}
					_currentIFrame.setTitle(title);

			        }
			      });
		} catch (SchemaCreationException e1) {
			e1.printStackTrace();
		} catch (DocumentCreationException e1) {
			e1.printStackTrace();
		}
		jf.setVisible(true);
		jf.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				jf.setVisible(false);
				jf.dispose();
				System.exit(0);
			}
		});
	}

	private static void addLanugages(JMenu languageMenu) {
		JMenuItem mi;
		languageMenu.add(mi = new JMenuItem("German"));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				changeLanguage("de");
			}
		});
		languageMenu.add(mi = new JMenuItem("English"));
		mi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				changeLanguage("en");
			}
		});
	}

	private static void changeLanguage(String language) {
		if (_currentEditorPanel != null) {
			Document dom = _currentEditorPanel.getDOM();
			dom.setLanguage(language);
			TypeVisualizerFactory.getInstance().releaseCache(_currentEditorPanel.getDOM());
			_currentEditorPanel = new EditorPanel(dom.getRootType(), null);
			_currentIFrame.setContentPane(_currentEditorPanel);
		}
	}
	private static void saveDOM() {
		//save your dom here
		try {
			String xml = _dom.serialize().toString();
			System.out.println(xml);
			_dom.getController().hasChanged(false);
		} catch (ValidationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
}