package com.jaxfront.demo;

import java.awt.BorderLayout;
import java.net.URL;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import com.jaxfront.core.dom.DOMBuilder;
import com.jaxfront.core.dom.DOMChangeEvent;
import com.jaxfront.core.dom.DOMChangeListener;
import com.jaxfront.core.dom.Document;
import com.jaxfront.core.dom.DocumentCreationException;
import com.jaxfront.core.schema.SchemaCreationException;
import com.jaxfront.core.schema.ValidationException;
import com.jaxfront.core.util.URLHelper;
import com.jaxfront.swing.ui.editor.EditorPanel;
import com.jaxfront.swing.ui.tools.GenericAction;

public class SimpleIntegration extends JFrame implements DOMChangeListener {
	
	private Document _dom;
	
	public static void main(String[] args) {
		new SimpleIntegration();
	}
	public SimpleIntegration() {
		initialize();
	}
	private void initialize() {
		getContentPane().setLayout(new BorderLayout());
		
		//Build JAXFront DOM
		URL xsdURL = URLHelper.getUserURL("examples/purchaseOrder/po.xsd");
		URL xmlURL = URLHelper.getUserURL("examples/purchaseOrder/po.xml");
		URL xuiURL = URLHelper.getUserURL("examples/purchaseOrder/po.xui");
		try {
			_dom = DOMBuilder.getInstance().build("context-SimpleIntegration", xsdURL, xmlURL, xuiURL, null);
			_dom.addDOMChangeListener(this);
			
			EditorPanel jaxfrontPanel = new EditorPanel(_dom.getRootType(), this);
			getContentPane().add(jaxfrontPanel, BorderLayout.CENTER);
			
			//Button control area
			JPanel buttonControlPanel = new JPanel(new BorderLayout());
			GenericAction saveAction = new GenericAction("save", this, "saveDOM");
			JButton saveButton = new JButton(saveAction);
			buttonControlPanel.add(saveButton, BorderLayout.EAST);
			
			getContentPane().add(buttonControlPanel, BorderLayout.SOUTH);
			
		} catch (SchemaCreationException e) {
			System.out.println("Unable to create XSD for: " + xsdURL.toExternalForm());
		} catch (DocumentCreationException e) {
			System.out.println("Unable to create DOM for: " + xsdURL.toExternalForm());
		}
		
		setSize(600, 600);
		setVisible(true);
		
	}
	public void domChanged(DOMChangeEvent event) {
		System.out.println("DOMChangeEvent occured: " + event.toString());
	}
	public void saveDOM() {
		//will be called if user clicks the save button in the button control area
		try {
			StringBuffer xml = _dom.serialize();
			System.out.println(xml);
		} catch (ValidationException e) {
		}
		
	}
}
