package com.jaxfront.demo;

/*-- 

 Copyright (C) 2001-2004 by xcentric technology & consulting GmbH. All rights reserved.

 This software is the confidential and proprietary information of xcentric technology & 
 consulting GmbH ("Confidential Information"). You shall not disclose such Confidential 
 Information and shall use it only in accordance with the terms of the license agreement
 you entered into with xcentric.

 www.jaxfront.com

 */
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.net.URL;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.JToolBar;
import javax.swing.LookAndFeel;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.border.EmptyBorder;

import com.jaxfront.core.dom.DOMBuilder;
import com.jaxfront.core.dom.DOMChangeEvent;
import com.jaxfront.core.dom.DOMChangeListener;
import com.jaxfront.core.dom.Document;
import com.jaxfront.core.help.HelpEvent;
import com.jaxfront.core.help.HelpListener;
import com.jaxfront.core.images.SystemImages;
import com.jaxfront.core.ui.TypeVisualizerFactory;
import com.jaxfront.core.util.JAXFrontProperties;
import com.jaxfront.core.util.LicenseErrorException;
import com.jaxfront.core.util.TextTools;
import com.jaxfront.core.util.URLHelper;
import com.jaxfront.core.util.io.BrowserControl;
import com.jaxfront.core.util.io.cache.XUICache;
import com.jaxfront.pdf.PDFGenerator;
import com.jaxfront.swing.ui.beans.TypeTable;
import com.jaxfront.swing.ui.editor.EditorPanel;
import com.jaxfront.swing.ui.editor.ShowXMLDialog;
import com.jaxfront.swing.ui.tools.FontTools;
import com.jaxfront.swing.ui.wrapper.JAXJSplitPane;
import com.jaxfront.xuieditor.XUIEditor;
import com.jaxfront.xuieditor.visualizers.XUIDebugView;
import com.l2fprod.gui.plaf.skin.Skin;
import com.l2fprod.gui.plaf.skin.SkinLookAndFeel;

/*****************************************************************************************************************************************************
 * JAXFront Integration Example and use of the EditorPanel API.
 * 
 * @version 1.60
 ****************************************************************************************************************************************************/
public class TestFrame extends JFrame implements WindowListener, HelpListener, DOMChangeListener {
	class ChangeLookAndFeelAction extends AbstractAction {
		LookAndFeel _lookAndFeel;
		String _packageName;

		public ChangeLookAndFeelAction(String name, String packageName) {
			super(name);
			_packageName = packageName;
		}

		public ChangeLookAndFeelAction(String name, LookAndFeel lookAndFeel) {
			super(name);
			_lookAndFeel = lookAndFeel;
		}

		public void actionPerformed(ActionEvent e) {
			try {
				if (_packageName != null) {
					Skin theSkinToUse = SkinLookAndFeel.loadThemePack(_packageName);
					SkinLookAndFeel.setSkin(theSkinToUse);
					changeLF(new SkinLookAndFeel());
				} else {
					changeLF(_lookAndFeel);
				}
			} catch (Exception e1) {
				e1.printStackTrace();
			}
		}
	}
	private final static String APPLICATION_TITLE = "JAXFront Demo";
	private final static int WINDOW_HEIGHT = 768;
	private final static int WINDOW_WIDTH = 1024;
	private final static int DIVIDER_LOCATION = 550;
	// My Example
	private static URL MY_EXAMPLE_XSD_URL = URLHelper.getUserURL("examples/purchaseOrder/po.xsd");
	private static URL MY_EXAMPLE_XML_URL = URLHelper.getUserURL("examples/purchaseOrder/po.xml");
	private static URL MY_EXAMPLE_XUI_URL = URLHelper.getUserURL("examples/purchaseOrder/po.xui");
	private static String _title;
	private static ImageIcon _imageIcon;
	// Misc. Components
	private JTabbedPane _tabbedPane;
	private JPanel _helpPanel;
	private JPanel _errorHolderPanel;
	private JPanel _unlockedFieldsPanel;
	private Document _currentDom;
	private JPanel _centerPanel;
	private JSplitPane _splitPane;
	private EditorPanel _editor;
	private int _exampleIndex;
	private String _currentLanguage = "en"; // english as default
	private String _exampleURL;
	private JLabel _loadingLabel;
	// Actions
	private AbstractAction _changeLFToxicAction;
	private AbstractAction _germanLanguageAction;
	private AbstractAction _englishLanguageAction;
	private AbstractAction _frenchLanguageAction;
	private AbstractAction _italianLanguageAction;
	private AbstractAction _serializeAction;
	private AbstractAction _example1Action;
	private AbstractAction _example2Action;
	private AbstractAction _example3Action;
	private AbstractAction _printAction;
	private AbstractAction _validateAction;
	private AbstractAction _lockAction;
	private AbstractAction _unLockAction;
	private AbstractAction _exitAction;
	private AbstractAction _myExampleAction;
	private AbstractAction _xuiEditorAction;
	private AbstractAction _demoSuiteAction;
	private AbstractAction _purchaseOrderAction;
	private AbstractAction _diseaseFormAction;
	private AbstractAction _expenseReportAction;
	private AbstractAction _claimAdviceAction;
	private AbstractAction _mobileAppAction;
	private AbstractAction _createNewAction;
	private AbstractAction _releaseXUICache;
	private ExampleHeader _exampleHeader;

	public TestFrame() {
		super();
		init();
	}

	public void domChanged(DOMChangeEvent event) {
		if (event.isAboutToValidate() && _splitPane != null && _currentDom.getController().getErrorController().hasValidationErrors()) {
			_splitPane.setDividerLocation(0.7);
		}
	}

	private void init() {
		try {
			// default l&f
			try {
				Skin theSkinToUse = SkinLookAndFeel.loadThemePack("themes/cellshadedthemepack.zip");
				SkinLookAndFeel.setSkin(theSkinToUse);
				changeLF(new SkinLookAndFeel());
			} catch (Throwable t) {}
			// init frame
			if (_title == null) {
				_title = "Purchase Order Example (po.xsd)";
			}
			setTitle(APPLICATION_TITLE);
			setIconImage(JAXFrontProperties.getImage(SystemImages.ICON_XCENTRIC));
			getContentPane().setLayout(new BorderLayout());
			// initialize actions and gui components
			initActions();
			initMenuBar();
			initToolBar();
			_centerPanel = new JPanel(new BorderLayout());
			_centerPanel.setBorder(new EmptyBorder(10, 10, 10, 10));
			_loadingLabel = new JLabel("loading UI...");
			_loadingLabel.setFont(FontTools.getBoldFont(14));
			_imageIcon = JAXFrontProperties.getImageIcon("examples/demo-suite/images/jaxfront.gif");
			setInformation(_title, _imageIcon);
			_tabbedPane = new JTabbedPane();
			_tabbedPane.setBorder(new EmptyBorder(0, 10, 10, 10));
			_helpPanel = new JPanel();
			_errorHolderPanel = new JPanel(new BorderLayout());
			_unlockedFieldsPanel = new JPanel(new BorderLayout());
			_tabbedPane.addTab("Hints & Errors", _errorHolderPanel);
			_tabbedPane.addTab("Help", _helpPanel);
			_tabbedPane.addTab("Lock", _unlockedFieldsPanel);
			_splitPane = new JAXJSplitPane(JSplitPane.VERTICAL_SPLIT);
			_splitPane.setDividerSize(8);
			_splitPane.setOneTouchExpandable(true);
			_splitPane.setContinuousLayout(true);
			_splitPane.setDividerLocation(0.8);
			_splitPane.setBorder(null);
			_splitPane.setTopComponent(_centerPanel);
			_splitPane.setBottomComponent(_tabbedPane);
			getContentPane().add(_splitPane, BorderLayout.CENTER);
			addWindowListener(this);
			setTitle(_title);
			setSize(WINDOW_WIDTH, WINDOW_HEIGHT);
			setVisible(true);
			example3();
		} catch (LicenseErrorException licEx) {
			licEx.showLicenseDialog(this);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	private void setInformation(String title, ImageIcon icon) {
		icon = null;
		if (_exampleHeader != null) _centerPanel.remove(_exampleHeader);
		if (icon == null) icon = JAXFrontProperties.getImageIcon("examples/demo-suite/images/jaxfront.gif");
		_exampleHeader = new ExampleHeader(this, title, icon, getXSDURL());
		_exampleHeader.setBorder(BorderFactory.createBevelBorder(2));
		_centerPanel.add(_exampleHeader, BorderLayout.NORTH);
		_centerPanel.updateUI();
	}

	public String getXSDURL() {
		if (MY_EXAMPLE_XSD_URL == null) return null;
		return MY_EXAMPLE_XSD_URL.toExternalForm();
	}

	public static void main(String[] args) {
		// JAXFrontProperties.getInstance().setUseEmbeddedNLS(true);
		try {
			Skin theSkinToUse = SkinLookAndFeel.loadThemePack("opusOSOlivethemepack.zip");
			SkinLookAndFeel.setSkin(theSkinToUse);
			UIManager.setLookAndFeel(new SkinLookAndFeel());
		} catch (Throwable t) {}
		for (int a = 0; a < args.length; a++) {
			String arg = args[a];
			if (arg.endsWith("xsd")) {
				MY_EXAMPLE_XSD_URL = URLHelper.getUserURL(arg);
			} else if (arg.endsWith("xml")) {
				MY_EXAMPLE_XML_URL = URLHelper.getUserURL(arg);
			} else if (arg.endsWith("xui")) {
				MY_EXAMPLE_XUI_URL = URLHelper.getUserURL(arg);
			} else if (arg.startsWith("title=")) {
				_title = arg.substring(6, arg.length());
			} else if (arg.startsWith("image=")) {
				_imageIcon = JAXFrontProperties.getImageIcon(arg.substring(6, arg.length()));
			}
		}
		new TestFrame();
	}

	private void changeLF(LookAndFeel lookAndFeel) {
		try {
			UIManager.setLookAndFeel(lookAndFeel);
			SwingUtilities.updateComponentTreeUI(this);
			// this.pack();
		} catch (UnsupportedLookAndFeelException e) {}
	}

	private void initActions() {
		_releaseXUICache = new AbstractAction("Release XUI Cache", JAXFrontProperties.getImageIcon(SystemImages.ICON_DELETE)) {
			public void actionPerformed(ActionEvent e) {
				releaseXUICache();
			}
		};
		_germanLanguageAction = new AbstractAction("German", JAXFrontProperties.getImageIcon(SystemImages.ICON_XUI_DEFINITION_GLOBAL)) {
			public void actionPerformed(ActionEvent e) {
				setLanguage("de");
			}
		};
		_englishLanguageAction = new AbstractAction("English", JAXFrontProperties.getImageIcon(SystemImages.ICON_XUI_DEFINITION_GLOBAL)) {
			public void actionPerformed(ActionEvent e) {
				setLanguage("en");
			}
		};
		_frenchLanguageAction = new AbstractAction("French", JAXFrontProperties.getImageIcon(SystemImages.ICON_XUI_DEFINITION_GLOBAL)) {
			public void actionPerformed(ActionEvent e) {
				setLanguage("fr");
			}
		};
		_italianLanguageAction = new AbstractAction("Italian", JAXFrontProperties.getImageIcon(SystemImages.ICON_XUI_DEFINITION_GLOBAL)) {
			public void actionPerformed(ActionEvent e) {
				setLanguage("it");
			}
		};
		_serializeAction = new AbstractAction("Serialize", JAXFrontProperties.getImageIcon(SystemImages.ICON_EDIT_XML)) {
			public void actionPerformed(ActionEvent e) {
				serialize();
			}
		};
		_lockAction = new AbstractAction("Lock", JAXFrontProperties.getImageIcon(SystemImages.ICON_LOCK)) {
			public void actionPerformed(ActionEvent e) {
				lock();
			}
		};
		_unLockAction = new AbstractAction("Unlock", JAXFrontProperties.getImageIcon(SystemImages.ICON_UNLOCK)) {
			public void actionPerformed(ActionEvent e) {
				unLock();
			}
		};
		_example1Action = new AbstractAction("Example 1", JAXFrontProperties.getImageIcon("examples/demo-suite/images/xsd.gif")) {
			public void actionPerformed(ActionEvent e) {
				example1();
			}
		};
		_example2Action = new AbstractAction("Example 2 (xsd + xml)", JAXFrontProperties.getImageIcon("examples/demo-suite/images/xml.gif")) {
			public void actionPerformed(ActionEvent e) {
				example2();
			}
		};
		_example3Action = new AbstractAction("Example 3 (xsd + xml + xui)", JAXFrontProperties.getImageIcon("examples/demo-suite/images/xui.gif")) {
			public void actionPerformed(ActionEvent e) {
				example3();
			}
		};
		_myExampleAction = new AbstractAction("My own example", JAXFrontProperties.getImageIcon(SystemImages.ICON_NEW_FILE)) {
			public void actionPerformed(ActionEvent e) {
				myExample();
			}
		};
		_printAction = new AbstractAction("Print PDF", JAXFrontProperties.getImageIcon(SystemImages.ICON_PDF)) {
			public void actionPerformed(ActionEvent e) {
				print();
			}
		};
		_validateAction = new AbstractAction("Validate", JAXFrontProperties.getImageIcon(SystemImages.ICON_VALIDATE)) {
			public void actionPerformed(ActionEvent e) {
				validateXML();
			}
		};
		_exitAction = new AbstractAction("Exit", JAXFrontProperties.getImageIcon(SystemImages.ICON_CLOSE)) {
			public void actionPerformed(ActionEvent e) {
				exit();
			}
		};
		_xuiEditorAction = new AbstractAction("Open XUIEditor", JAXFrontProperties.getImageIcon(SystemImages.ICON_XCENTRIC)) {
			public void actionPerformed(ActionEvent e) {
				openXUIEditor();
			}
		};
		_demoSuiteAction = new AbstractAction("JAXFront Demo Suite", JAXFrontProperties.getImageIcon(SystemImages.ICON_XCENTRIC)) {
			public void actionPerformed(ActionEvent e) {
				MY_EXAMPLE_XSD_URL = URLHelper.getUserURL("examples/demo-suite/jaxfront-test-schema.xsd");
				MY_EXAMPLE_XML_URL = URLHelper.getUserURL("examples/demo-suite/jaxfront-test-schema.xml");
				MY_EXAMPLE_XUI_URL = URLHelper.getUserURL("examples/demo-suite/jaxfront-test-schema.xui");
				setInformation("JAXFront Demo Suite", JAXFrontProperties.getImageIcon("examples/demo-suite/images/jaxfront.gif"));
			}
		};
		_purchaseOrderAction = new AbstractAction("Purchase Order", JAXFrontProperties.getImageIcon(SystemImages.ICON_XCENTRIC)) {
			public void actionPerformed(ActionEvent e) {
				MY_EXAMPLE_XSD_URL = URLHelper.getUserURL("examples/purchaseOrder/po.xsd");
				MY_EXAMPLE_XML_URL = URLHelper.getUserURL("examples/purchaseOrder/po.xml");
				MY_EXAMPLE_XUI_URL = URLHelper.getUserURL("examples/purchaseOrder/po.xui");
				setInformation("Purchase Order Example", JAXFrontProperties.getImageIcon("examples/purchaseOrder/po.jpg"));
			}
		};
		_diseaseFormAction = new AbstractAction("Disease Form", JAXFrontProperties.getImageIcon(SystemImages.ICON_XCENTRIC)) {
			public void actionPerformed(ActionEvent e) {
				MY_EXAMPLE_XSD_URL = URLHelper.getUserURL("examples/diseaseForm/df.xsd");
				MY_EXAMPLE_XML_URL = URLHelper.getUserURL("examples/diseaseForm/df.xml");
				MY_EXAMPLE_XUI_URL = URLHelper.getUserURL("examples/diseaseForm/df.xui");
				setInformation("Disease Form Example", JAXFrontProperties.getImageIcon("examples/diseaseForm/df.jpg"));
			}
		};
		_expenseReportAction = new AbstractAction("Expense Report", JAXFrontProperties.getImageIcon(SystemImages.ICON_XCENTRIC)) {
			public void actionPerformed(ActionEvent e) {
				MY_EXAMPLE_XSD_URL = URLHelper.getUserURL("examples/expenseReport/er.xsd");
				MY_EXAMPLE_XML_URL = URLHelper.getUserURL("examples/expenseReport/er.xml");
				MY_EXAMPLE_XUI_URL = URLHelper.getUserURL("examples/expenseReport/er.xui");
				setInformation("Expense Report Example", JAXFrontProperties.getImageIcon("examples/expenseReport/er.jpg"));
			}
		};
		_claimAdviceAction = new AbstractAction("Claim Advice", JAXFrontProperties.getImageIcon(SystemImages.ICON_XCENTRIC)) {
			public void actionPerformed(ActionEvent e) {
				MY_EXAMPLE_XSD_URL = URLHelper.getUserURL("examples/claimAdvice/claimAdvice.xsd");
				MY_EXAMPLE_XML_URL = null; // URLHelper.getUserURL("../JAXFront Mobi Beans/resources/v4.0/saMeldungV4.0.xml");
				MY_EXAMPLE_XUI_URL = URLHelper.getUserURL("examples/claimAdvice/claimAdvice.xui");
				JAXFrontProperties.getInstance().setUserProperty("CURRENT_MELDUNGS_NUMMER", "M-4322-200987");
				JAXFrontProperties.getInstance().setUserProperty("CURRENT_RESPONSIBLE_GA", "Gossau (SG)");
				JAXFrontProperties.getInstance().setUserProperty("PRINT_DATE_USER", "02.08.2005 14:30, Franziska Stebler (U109006)");
				JAXFrontProperties.getInstance().setUserProperty("CURRENT_USER", "spo");
				_currentLanguage = "de";
				setInformation("Claim Advice Example", JAXFrontProperties.getImageIcon("examples/claimAdvice/images/claimAdvice.gif"));
			}
		};
		_mobileAppAction = new AbstractAction("Mobile Phone Subscription", JAXFrontProperties.getImageIcon(SystemImages.ICON_XCENTRIC)) {
			public void actionPerformed(ActionEvent e) {
				MY_EXAMPLE_XSD_URL = URLHelper.getUserURL("examples/mobileApplication/subscription.xsd");
				MY_EXAMPLE_XML_URL = URLHelper.getUserURL("examples/mobileApplication/subscription.xml");
				MY_EXAMPLE_XUI_URL = URLHelper.getUserURL("examples/mobileApplication/subscription.xui");
				setInformation("Mobile Phone Subscription Example", JAXFrontProperties.getImageIcon("examples/mobileApplication/mobile-phone-small.gif"));
			}
		};
		_createNewAction = new AbstractAction("Create New", JAXFrontProperties.getImageIcon(SystemImages.ICON_NEW)) {
			public void actionPerformed(ActionEvent e) {
				MY_EXAMPLE_XML_URL = null;
				myExample();
			}
		};
		_englishLanguageAction.setEnabled(false);
		_germanLanguageAction.setEnabled(false);
		_frenchLanguageAction.setEnabled(false);
		_serializeAction.setEnabled(false);
		_printAction.setEnabled(false);
		_validateAction.setEnabled(false);
	}

	public void myExample() {
		if (MY_EXAMPLE_XSD_URL == null) example3();
		else {
			try {
				_centerPanel.add(_loadingLabel, BorderLayout.CENTER);
				_currentDom = DOMBuilder.getInstance().build(null, MY_EXAMPLE_XSD_URL, MY_EXAMPLE_XML_URL, MY_EXAMPLE_XUI_URL, null);
				_currentDom.getGlobalDefinition().setIsUsingButtonBar(false);
				_currentDom.getGlobalDefinition().setIsUsingStatusBar(false);
				_currentDom.addDOMChangeListener(this);
				if (_editor != null) _editor.selectNode((com.jaxfront.core.type.Type) null);
				visualizeDOM();
				setApplicationTitle("Individual Rendering (based on XUI)");
			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}
	}

	private void initMenuBar() {
		JMenuBar menuBar = new JMenuBar();
		JMenu applicationMenu = new JMenu("Application");
		applicationMenu.add(_releaseXUICache);
		applicationMenu.add(_exitAction);
		JMenu caseMenu = new JMenu("Case");
		JRadioButtonMenuItem demoSuite = new JRadioButtonMenuItem(_demoSuiteAction);
		JRadioButtonMenuItem purchaseOrder = new JRadioButtonMenuItem(_purchaseOrderAction);
		JRadioButtonMenuItem diseaseForm = new JRadioButtonMenuItem(_diseaseFormAction);
		JRadioButtonMenuItem expenseReport = new JRadioButtonMenuItem(_expenseReportAction);
		JRadioButtonMenuItem claimAdvice = new JRadioButtonMenuItem(_claimAdviceAction);
		JRadioButtonMenuItem mobileApp = new JRadioButtonMenuItem(_mobileAppAction);
		purchaseOrder.setSelected(true);
		caseMenu.add(purchaseOrder);
		caseMenu.add(demoSuite);
		caseMenu.add(diseaseForm);
		caseMenu.add(expenseReport);
		caseMenu.add(claimAdvice);
		caseMenu.add(mobileApp);
		ButtonGroup group = new ButtonGroup();
		group.add(demoSuite);
		group.add(purchaseOrder);
		group.add(diseaseForm);
		group.add(expenseReport);
		group.add(claimAdvice);
		group.add(mobileApp);
		JMenu exampleMenu = new JMenu("Examples");
		// exampleMenu.add(caseMenu);
		exampleMenu.addSeparator();
		exampleMenu.add(_example1Action);
		exampleMenu.add(_example2Action);
		exampleMenu.add(_example3Action);
		JMenu lookAndFeelMenu = new JMenu("Look & Feel");
		JRadioButtonMenuItem lookAndFeel_1 = new JRadioButtonMenuItem(new ChangeLookAndFeelAction("Opus OSO Olive", "themes/opusOSOlivethemepack.zip"));
		JRadioButtonMenuItem lookAndFeel_2 = new JRadioButtonMenuItem(new ChangeLookAndFeelAction("Quick Silver", "themes/quickSilverRthemepack.zip"));
		JRadioButtonMenuItem lookAndFeel_3 = new JRadioButtonMenuItem(new ChangeLookAndFeelAction("Cell Shaded", "themes/cellshadedthemepack.zip"));
		JRadioButtonMenuItem lookAndFeel_4 = new JRadioButtonMenuItem(new ChangeLookAndFeelAction("Gorilla", "themes/gorillathemepack.zip"));
		JRadioButtonMenuItem lookAndFeel_5 = new JRadioButtonMenuItem(new ChangeLookAndFeelAction("BBJ", "themes/bbjthemepack.zip"));
		// JRadioButtonMenuItem lookAndFeel_7 = new JRadioButtonMenuItem(new ChangeLookAndFeelAction("Napkin", new NapkinLookAndFeel()));
		lookAndFeel_3.setSelected(true);
		lookAndFeelMenu.add(lookAndFeel_1);
		lookAndFeelMenu.add(lookAndFeel_2);
		lookAndFeelMenu.add(lookAndFeel_3);
		lookAndFeelMenu.add(lookAndFeel_4);
		lookAndFeelMenu.add(lookAndFeel_5);
		// lookAndFeelMenu.add(lookAndFeel_7);
		ButtonGroup group2 = new ButtonGroup();
		group2.add(lookAndFeel_1);
		group2.add(lookAndFeel_2);
		group2.add(lookAndFeel_3);
		group2.add(lookAndFeel_4);
		group2.add(lookAndFeel_5);
		// group2.add(lookAndFeel_7);
		JMenu editMenu = new JMenu("Edit");
		editMenu.add(_createNewAction);
		editMenu.addSeparator();
		editMenu.add(_serializeAction);
		editMenu.add(_validateAction);
		editMenu.addSeparator();
		editMenu.add(_lockAction);
		editMenu.add(_unLockAction);
		editMenu.addSeparator();
		editMenu.add(_xuiEditorAction);
		editMenu.addSeparator();
		editMenu.add(lookAndFeelMenu);
		JMenu languageMenu = new JMenu("Language");
		languageMenu.add(_englishLanguageAction);
		languageMenu.add(_germanLanguageAction);
		languageMenu.add(_frenchLanguageAction);
		languageMenu.add(_italianLanguageAction);
		menuBar.add(applicationMenu);
		menuBar.add(exampleMenu);
		menuBar.add(editMenu);
		menuBar.add(languageMenu);
		setJMenuBar(menuBar);
	}

	private void initToolBar() {
		JToolBar toolBar = new JToolBar();
		toolBar.add(_example1Action);
		toolBar.add(_example2Action);
		toolBar.add(_example3Action);
		toolBar.addSeparator();
		toolBar.add(_createNewAction);
		toolBar.add(_serializeAction);
		toolBar.add(_validateAction);
		toolBar.addSeparator();
		toolBar.add(_printAction);
		toolBar.addSeparator();
		toolBar.add(_xuiEditorAction);
		getContentPane().add(toolBar, BorderLayout.NORTH);
	}

	private void validateXML() {
		if (_currentDom != null) {
			_currentDom.validate();
		}
	}

	private void print() {
		if (_currentDom != null) {
			ByteArrayOutputStream bos = PDFGenerator.getInstance().print(_currentDom);
			if (bos != null) {
				try {
					String tempPDFName = "c:\\temp\\test.pdf";
					FileOutputStream fos = new FileOutputStream(tempPDFName);
					bos.writeTo(fos);
					fos.close();
					BrowserControl.displayURL(tempPDFName);
				} catch (Throwable t) {
					t.printStackTrace();
				}
			}
		}
	}

	private String getCurrentLanguage() {
		return _currentLanguage;
	}

	private void visualizeDOM() {
		// set language
		_unlockedFieldsPanel.removeAll();
		_unlockedFieldsPanel.updateUI();
		_currentDom.getGlobalDefinition().setLanguage(getCurrentLanguage());
		com.jaxfront.core.type.Type lastSelectedType = null;
		if (_editor != null && _editor.getSelectedTreeNode() != null) {
			lastSelectedType = _editor.getSelectedTreeNode().getType();
		}
		TypeVisualizerFactory.getInstance().releaseCache(_currentDom);
		EditorPanel editorPanel = new EditorPanel(_currentDom.getRootType(), this);
		if (lastSelectedType != null) {
			editorPanel.selectNode(lastSelectedType);
		}
		editorPanel.setBorder(null);
		editorPanel.addHelpListener(this);
		JPanel validationErrorPanel = new JPanel(new BorderLayout());
		validationErrorPanel.setBorder(null);
		editorPanel.setTargetMessageTable(validationErrorPanel);
		_centerPanel.remove(_loadingLabel);
		if (_editor != null) _centerPanel.remove(_editor);
		_editor = editorPanel;
		_centerPanel.add(_editor, BorderLayout.CENTER);
		int lastDividerLocation = DIVIDER_LOCATION;
		if (_splitPane.getBottomComponent() != null) {
			// lastDividerLocation = _splitPane.getDividerLocation();
			_errorHolderPanel.removeAll();
			_errorHolderPanel.add(validationErrorPanel, BorderLayout.CENTER);
			_splitPane.validate();
		}
		// activate actions
		_englishLanguageAction.setEnabled(true);
		_germanLanguageAction.setEnabled(true);
		_frenchLanguageAction.setEnabled(true);
		_serializeAction.setEnabled(true);
		_printAction.setEnabled(true);
		_validateAction.setEnabled(true);
	}

	private void setLanguage(String language) {
		_currentLanguage = language;
		visualizeDOM();
	}

	private void serialize() {
		_currentDom.validate();
		System.out.println(_currentDom.isValid());
		ShowXMLDialog dialog = new ShowXMLDialog(_currentDom);
		dialog.prettyPrint();
		Dimension dialogDim = dialog.getSize();
		Dimension thisDim = getSize();
		int x = (thisDim.width - dialogDim.width) / 2;
		int y = (thisDim.height - dialogDim.height) / 2;
		if (getLocation().x > 0 || getLocation().y > 0) {
			x = x + getLocation().x;
			y = y + getLocation().y;
		}
		dialog.setLocation(((x > 0) ? x : 0), ((y > 0) ? y : 0));
		dialog.setVisible(true);
	}

	private void lock() {
		String[] fieldToUnlock = new String[] { "/SchadenMeldung/Ereignis/Hergang", "/SchadenMeldung/WeiteresVorgehenNotizen[list]" };
		if (_currentDom != null && _currentDom.getRootType().getName().equals("purchaseOrder")) {
			// fieldToUnlock = new String[] {"/purchaseOrder/@orderDate", "/purchaseOrder/item[list]"};
			fieldToUnlock = new String[] { "/purchaseOrder/@orderDate", "/purchaseOrder/paymentMethod" };
		}
		_currentDom.lock(fieldToUnlock);
		_unlockedFieldsPanel.removeAll();
		_unlockedFieldsPanel.setFocusable(false);
		JScrollPane scrollPane = new JScrollPane(new TypeTable(_currentDom, fieldToUnlock));
		scrollPane.setFocusable(false);
		scrollPane.getViewport().setFocusable(false);
		scrollPane.getVerticalScrollBar().setFocusable(false);
		scrollPane.getHorizontalScrollBar().setFocusable(false);
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		_unlockedFieldsPanel.add(scrollPane, BorderLayout.CENTER);
		_unlockedFieldsPanel.updateUI();
		_tabbedPane.setSelectedIndex(2);
	}

	private void unLock() {
		_currentDom.unlock();
		_unlockedFieldsPanel.removeAll();
		_unlockedFieldsPanel.updateUI();
	}

	private void setApplicationTitle(String currentFileName) {
		setTitle(APPLICATION_TITLE + " - " + currentFileName);
	}

	private void example1() {
		_exampleIndex = 0;
		_centerPanel.add(_loadingLabel, BorderLayout.CENTER);
		URL url = URLHelper.getUserURL("examples/purchaseOrder/po.xsd");
		URL xmlUrl = null;
		URL xuiUrl = null;
		if (MY_EXAMPLE_XSD_URL != null) url = MY_EXAMPLE_XSD_URL;
		try {
			_currentDom = DOMBuilder.getInstance().build(null, url, xmlUrl, xuiUrl, null);
			_currentDom.getGlobalDefinition().setIsUsingButtonBar(false);
			_currentDom.getGlobalDefinition().setIsUsingStatusBar(false);
			_currentDom.addDOMChangeListener(this);
			if (_editor != null) _editor.selectNode((com.jaxfront.core.type.Type) null);
			visualizeDOM();
			setApplicationTitle("Default Rendering (based on XML Schema only)");
		} catch (Exception ex) {
			ex.printStackTrace();
			String xsd = null;
			String xml = null;
			String xui = null;
			if (url != null) xsd = url.toExternalForm();
			if (xmlUrl != null) xml = xmlUrl.toExternalForm();
			if (xuiUrl != null) xui = xuiUrl.toExternalForm();
			JOptionPane.showConfirmDialog(this, "Unable to visualize: \n " + "xsd - " + xsd + "\n" + "xml - " + xml + "\n" + "xui - " + xui + "\n");
		}
	}

	private void example2() {
		_exampleIndex = 1;
		_centerPanel.add(_loadingLabel, BorderLayout.CENTER);
		URL url = URLHelper.getUserURL("examples/purchaseOrder/po.xsd");
		URL xmlUrl = URLHelper.getUserURL("examples/purchaseOrder/po.xml");
		URL xuiUrl = null;
		String root = null;
		if (MY_EXAMPLE_XSD_URL != null) url = MY_EXAMPLE_XSD_URL;
		if (MY_EXAMPLE_XML_URL != null) xmlUrl = MY_EXAMPLE_XML_URL;
		try {
			_currentDom = DOMBuilder.getInstance().build(null, url, xmlUrl, xuiUrl, root);
			_currentDom.getGlobalDefinition().setIsUsingButtonBar(false);
			_currentDom.getGlobalDefinition().setIsUsingStatusBar(false);
			_currentDom.addDOMChangeListener(this);
			if (_editor != null) _editor.selectNode((com.jaxfront.core.type.Type) null);
			visualizeDOM();
			setApplicationTitle("Default Rendering including XML data (based on XML Schema only)");
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	private void example3() {
		_exampleIndex = 2;
		_centerPanel.add(_loadingLabel, BorderLayout.CENTER);
		URL url = URLHelper.getUserURL("examples/purchaseOrder/po.xsd");
		URL xmlUrl = URLHelper.getUserURL("examples/purchaseOrder/po.xml");
		URL xuiUrl = URLHelper.getUserURL("examples/purchaseOrder/po.xui");
		String root = null;
		if (MY_EXAMPLE_XSD_URL != null) url = MY_EXAMPLE_XSD_URL;
		if (MY_EXAMPLE_XML_URL != null) xmlUrl = MY_EXAMPLE_XML_URL;
		if (MY_EXAMPLE_XUI_URL != null) xuiUrl = MY_EXAMPLE_XUI_URL;
		try {
			_currentDom = DOMBuilder.getInstance().build(null, url, xmlUrl, xuiUrl, root);
			_currentDom.getGlobalDefinition().setLanguage("de");
			_currentDom.getGlobalDefinition().setIsUsingButtonBar(false);
			_currentDom.getGlobalDefinition().setIsUsingStatusBar(false);
			_currentDom.addDOMChangeListener(this);
			if (_editor != null) _editor.selectNode((com.jaxfront.core.type.Type) null);
			visualizeDOM();
			setApplicationTitle("Individual Rendering (based on XUI)");
		} catch (Exception ex) {}
	}

	private void exit() {
		System.exit(0);
	}

	public void windowActivated(WindowEvent e) {
		XUIDebugView.setShowXUIDivergency(false);
	}

	public void windowClosed(WindowEvent e) {
		exit();
	}

	public void windowClosing(WindowEvent e) {
		exit();
	}

	public void windowDeactivated(WindowEvent e) {}

	public void windowDeiconified(WindowEvent e) {}

	public void windowIconified(WindowEvent e) {}

	public void windowOpened(WindowEvent e) {}

	public void openXUIEditor() {
		if (_currentDom != null) _currentDom.release();
		if (_editor != null) {
			_centerPanel.remove(_editor);
			_centerPanel.updateUI();
			_printAction.setEnabled(false);
			_serializeAction.setEnabled(false);
		}
		String xsd = "examples/purchaseOrder/po.xsd";
		String xml = "-XML=examples/purchaseOrder/po.xml";
		String xui = "-XUI=examples/purchaseOrder/po.xui";
		if (MY_EXAMPLE_XSD_URL != null) {
			xsd = MY_EXAMPLE_XSD_URL.toExternalForm();
			if (MY_EXAMPLE_XML_URL != null) xml = "-XML=" + MY_EXAMPLE_XML_URL.toExternalForm();
			if (MY_EXAMPLE_XUI_URL != null) xui = "-XUI=" + MY_EXAMPLE_XUI_URL.toExternalForm();
		}
		if (xsd != null) {
			xsd = TextTools.replace(xsd, "%20", " ");
			xsd = TextTools.replace(xsd, "file:/", "file:///");
		}
		if (xml != null) {
			xml = TextTools.replace(xml, "%20", " ");
			xml = TextTools.replace(xml, "file:/", "file:///");
		}
		if (xui != null) {
			xui = TextTools.replace(xui, "%20", " ");
			xui = TextTools.replace(xui, "file:/", "file:///");
		}
		String[] args = new String[] { xsd, xml, xui };
		XUIEditor.main(args);
		XUIEditor.setBlockFileActions(true);
	}

	public void showHelp(HelpEvent event) {
		_helpPanel.removeAll();
		String helpText = "Help (" + event.getHelpID() + "): " + ((com.jaxfront.core.type.Type) event.getSource()).getXPathLocation();
		JLabel helpLabel = new JLabel(helpText);
		_helpPanel.add(helpLabel);
		_helpPanel.updateUI();
	}

	private void releaseXUICache() {
		XUICache.getInstance().releaseCache();
	}
}