package com.jaxfront.demo.api;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import com.jaxfront.core.rule.Condition;
import com.jaxfront.core.rule.FormulaException;
import com.jaxfront.core.type.SimpleType;
import com.jaxfront.core.type.Type;

/**
 * Demonstrates how to implement an own condition. This condition checks if the underlying date is a future date.
 */
/*-- 

 Copyright (C) 2001 by xcentric technology & consulting GmbH.
 All rights reserved.

 This software is the confidential and proprietary information
 of xcentric technology & consulting GmbH ("Confidential Information"). 
 You shall not disclose such Confidential Information and shall use
 it only in accordance with the terms of the license agreement
 you entered into with xcentric.

 www.jaxfront.com

 */
public class OwnCondition extends Object implements Condition {
	Type _sourceType;

	public boolean isTrue(Type source) throws FormulaException {
		SimpleType simpleType = (SimpleType) _sourceType;
		String dateString = simpleType.getValue();
		try {
			SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd");
			Date inputDate = format.parse(dateString);
			Date currentDate = new Date();
			return inputDate.compareTo(currentDate) > 0;
		} catch (ParseException ex) {
			// do nothing
		}
		return false;
	}

	public boolean doesConcern(Type checkType) {
		return checkType == _sourceType;
	}

	public void setSourceType(Type sourceType) {
		_sourceType = sourceType;
	}
}
