package com.jaxfront.demo.ui.beans;

import java.awt.Color;
import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import com.jaxfront.core.help.HelpEvent;
import com.jaxfront.core.help.HelpListener;
import com.jaxfront.core.type.Type;
import com.jaxfront.core.ui.TypeVisualizerFactory;
import com.jaxfront.core.ui.Visualizer;
import com.jaxfront.swing.ui.layout.ListLayout;
import com.jaxfront.swing.ui.layout.PercentLayout;

/*-- 

 Copyright (C) 2001-2003 by xcentric technology & consulting GmbH. All rights reserved.

 This software is the confidential and proprietary information of xcentric technology & 
 consulting GmbH ("Confidential Information"). You shall not disclose such Confidential 
 Information and shall use it only in accordance with the terms of the license agreement
 you entered into with xcentric.

 www.jaxfront.com

 */
public class HelpVisualizer extends JPanel implements Visualizer, PropertyChangeListener, HelpListener {
	private Type _model;
	private JLabel _label;
	private JLabel _htmlPreview;
	private boolean _uiReleased;

	/**
	 * Default constructor.
	 */
	public HelpVisualizer() {
	}

	public void postInitialization() {
	}

	/**
	 * @see com.jaxfront.ui.Visualizer#getLabelText()
	 */
	public String getLabelText() {
		return "Example Help Visualizer";
	}

	/**
	 * @see com.jaxfront.ui.Visualizer#getModel()
	 */
	public Type getModel() {
		return _model;
	}

	/**
	 * @see com.jaxfront.ui.Visualizer#markError()
	 */
	public void markError() {
	}

	/**
	 * @see com.jaxfront.ui.Visualizer#populateModel()
	 */
	public void populateModel() {
		// do nothing
	}

	/**
	 * @see com.jaxfront.ui.Visualizer#populateView()
	 */
	public void populateView() {
		// do nothing
	}

	/**
	 * @see com.jaxfront.ui.Visualizer#release()
	 */
	public void release() {
		if (!_uiReleased) {
			_model.getDOM().getEditor().removeHelpListener(this);
		}
		_model = null;
		_uiReleased = true;
	}

	public boolean isViewReleased() {
		return _uiReleased;
	}

	/**
	 * @see com.jaxfront.ui.Visualizer#removeLabel()
	 */
	public void removeLabel() {
	}
	public int getRendererType() {
		return TypeVisualizerFactory.RENDERER_TYPE_SWING;
	}
	/**
	 * @see com.jaxfront.ui.Visualizer#resetError()
	 */
	public void resetError() {
	}

	/**
	 * @see com.jaxfront.ui.Visualizer#setApplicationRequired(boolean)
	 */
	public void setApplicationRequired(boolean required) {
	}

	/**
	 * @see com.jaxfront.ui.Visualizer#setEditable(boolean)
	 */
	public void setEditable(boolean enabled) {
	}

	/**
	 * @see com.jaxfront.ui.Visualizer#setLabelColor(java.awt.Color)
	 */
	public void setLabelColor(Color color) {
	}

	/**
	 * @see com.jaxfront.ui.Visualizer#setModel(com.jaxfront.types.Type)
	 */
	public void setModel(Type type) {
		_model = type;
		if (_model != null) {
			_model.getPropertyChangeSupport().addPropertyChangeListener(this);
			_model.getDOM().getEditor().addHelpListener(this);
			defaultBuild();
			populateView();
		}
	}

	/**
	 * @see com.jaxfront.ui.Visualizer#setRequired(boolean)
	 */
	public void setRequired(boolean required) {
	}

	/**
	 * @see com.jaxfront.ui.Visualizer#showLabel(boolean)
	 */
	public void showLabel(boolean show) {
	}

	private void defaultBuild() {
		PercentLayout percentLayout = new PercentLayout(1); // 1 forhorizontal
		PercentLayout.Constraints labelConstraint = new PercentLayout.Constraints(30, 1);
		PercentLayout.Constraints componentConstraint = new PercentLayout.Constraints(70, 1);
		setLayout(percentLayout);
		_label = new JLabel(getLabelText());
		JPanel holderPanel = new JPanel();
		holderPanel.setLayout(new ListLayout());
		JScrollPane scrollPanePreview = new JScrollPane();
		scrollPanePreview.setPreferredSize(new Dimension(10, 90));
		scrollPanePreview.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scrollPanePreview.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		_htmlPreview = new JLabel();
		scrollPanePreview.getViewport().add(_htmlPreview);
		holderPanel.add(scrollPanePreview);
		add(_label, labelConstraint);
		add(holderPanel, componentConstraint);
	}

	/**
	 * @see java.beans.PropertyChangeListener#propertyChange(java.beans.PropertyChangeEvent)
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getSource() == _model) {
			populateView();
		}
	}

	/*
	 * (non-Javadoc)
	 * @see com.jaxfront.core.ui.Visualizer#jumpTo()
	 */
	public void jumpTo() {
	}

	public void showHelp(HelpEvent event) {
		String text = "<HTML><BODY><table width=\"450\" cellpadding=\"0\" cellspacing=\"0\"><tr><td align=\"left\" " + "valign=\"top\"><table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\" bgcolor=\"#666666\">" + "<tr><td><b><font color=\"#FFFFFF\" size=\"5\" face=\"Arial\">Help for: "
				+ event.getHelpID() + "</font></b></td>" + "</tr></table><table width=\"100%\" cellpadding=\"0\" cellspacing=\"0\"><tr><td width=\"209\" " + "bgcolor=\"#E6E6E6\"><b><font face=\"Arial\" size=4 color=\"#666666\">Description</font></b></td>"
				+ "</tr><tr><td bgcolor=\"#FFFFFF\"><font color=\"#000000\" face=\"Arial\">This field is..." + "</font></td></tr><tr><td bgcolor=\"#E6E6E6\"><b><font color=\"#666666\" size=\"4\" face=\"Arial\">Source</font></b></td>"
				+ "</tr><tr><td bgcolor=\"#FFFFFF\"><font color=\"#000000\" face=\"Arial\">" + ((Type) event.getSource()).getXPathLocation() + "</font></td></tr><tr></tr></table></BODY></HTML>";
		_htmlPreview.setText(text);
	}

	public Visualizer getImplementation() {
		return this;
	}

	public Type getContext() {
		// TODO Auto-generated method stub
		return null;
	}

	public void setContext(Type type) {
	}

	public void setLabelText(String text) {
		// TODO Auto-generated method stub
	}

	public void setRenderedComponentType(RenderedComponentType renderedComponentType) {
		// TODO Auto-generated method stub
		
	}

	public RenderedComponentType getRenderedComponentType() {
		// TODO Auto-generated method stub
		return null;
	}

	public Object getRenderedComponent() {
		// TODO Auto-generated method stub
		return null;
	}
}
